/*
 * fasta_sequence_reader.cpp
 *
 *  Created on: 2009/06/19
 *      Author: shu
 */

#include <fstream>
#include <string>
#include <cctype>
#include <algorithm>
#include <functional>
#include "sequence.h"
#include "fasta_sequence_reader.h"

// create sequence
// return Sequence pointer, or if file is eof, return false.
SequencePtr FastaSequenceReader::Read
(
    // no parameter
)
{
  std::string name;     // got name from file
  std::string sequence = "";  // got sequence form file
  std::string line;
  size_t position;


  if (!in_.eof()) {
    line = last_line_;

    // then jump to the next header line
    while (!in_.eof() && (line.length() == 0 || line.at(0) != '>')) {
      std::getline(in_, line);
    }
    if (in_.eof()) return SequencePtr();

    // set name
    if (line.at(line.length() - 1) == '\r') {
      line = line.substr(0, line.length() - 1);
    }
    position = line.find_first_not_of("> ");
    if (position != std::string::npos) {
      name = line.substr(position);
    }

    // set sequence
    while (!in_.eof()) {
      std::getline(in_, line);
      if (line.length() != 0) {
        if (line.at(0) != '>') {
	  std::string::iterator left = 
            std::find_if(line.begin(), line.end(), std::not1(std::ptr_fun(isspace)));
	  
	  std::string::reverse_iterator right = 
            std::find_if(line.rbegin(), line.rend(), std::not1(std::ptr_fun(isspace)));
	  line = std::string(left, right.base());
          sequence += line;
        } else {
          break;
        }
      }
    }

    last_line_ = line;
    std::transform(sequence.begin(), sequence.end(), sequence.begin(), toupper);
    return SequencePtr(new Sequence(name, sequence));
  } else {
    return SequencePtr();
  }
}
